package com.ruoyi.web.controller.coffee;

import java.util.List;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.OrderRefund;
import com.ruoyi.system.service.IOrderRefundService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 订单退款Controller
 * 
 * @author ruoyi
 * @date 2022-04-28
 */
@RestController
@RequestMapping("/system/refund")
public class OrderRefundController extends BaseController
{
    @Autowired
    private IOrderRefundService orderRefundService;

    /**
     * 查询订单退款列表
     */
    @PreAuthorize("@ss.hasPermi('system:refund:list')")
    @GetMapping("/list")
    public TableDataInfo list(OrderRefund orderRefund)
    {
        startPage();
        List<OrderRefund> list = orderRefundService.selectOrderRefundList(orderRefund);
        return getDataTable(list);
    }

    /**
     * 获取订单退款详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:refund:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(orderRefundService.selectOrderRefundById(id));
    }

    /**
     * 新增订单退款
     */
    @PreAuthorize("@ss.hasPermi('system:refund:add')")
    @Log(title = "订单退款", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody OrderRefund orderRefund)
    {
        return toAjax(orderRefundService.insertOrderRefund(orderRefund));
    }

    /**
     * 修改订单退款
     */
    @PreAuthorize("@ss.hasPermi('system:refund:edit')")
    @Log(title = "订单退款", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody OrderRefund orderRefund)
    {
        return toAjax(orderRefundService.updateOrderRefund(orderRefund));
    }

    /**
     * 删除订单退款
     */
    @PreAuthorize("@ss.hasPermi('system:refund:remove')")
    @Log(title = "订单退款", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(orderRefundService.deleteOrderRefundByIds(ids));
    }
}
